@@ -1,9 +1,10 @@
 use crate::consts::{constant_simple, Constant};
 use crate::utils::span_help_and_lint;
 use if_chain::if_chain;
+use rustc::declare_lint_pass;
 use rustc::hir::*;
 use rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
-use rustc::{declare_lint_pass, declare_tool_lint};
+use rustc_session::declare_tool_lint;
 
 declare_clippy_lint! {
     /// **What it does:** Checks for `0.0 / 0.0`.
@@ -28,7 +29,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for ZeroDiv {
     fn check_expr(&mut self, cx: &LateContext<'a, 'tcx>, expr: &'tcx Expr) {
         // check for instances of 0.0/0.0
         if_chain! {
-            if let ExprKind::Binary(ref op, ref left, ref right) = expr.node;
+            if let ExprKind::Binary(ref op, ref left, ref right) = expr.kind;
             if let BinOpKind::Div = op.node;
             // TODO - constant_simple does not fold many operations involving floats.
             // That's probably fine for this lint - it's pretty unlikely that someone would
