@@ -1,5 +1,13 @@
+// run-rustfix
+
 #![warn(clippy::all)]
-#![allow(clippy::blacklisted_name, clippy::no_effect, redundant_semicolon, unused_assignments)]
+#![allow(
+    clippy::blacklisted_name,
+    clippy::no_effect,
+    clippy::redundant_clone,
+    redundant_semicolon,
+    unused_assignments
+)]
 
 struct Foo(u32);
 
@@ -40,6 +48,15 @@ fn slice() {
     foo.swap(0, 1);
 }
 
+fn unswappable_slice() {
+    let foo = &mut [vec![1, 2], vec![3, 4]];
+    let temp = foo[0][1];
+    foo[0][1] = foo[1][0];
+    foo[1][0] = temp;
+
+    // swap(foo[0][1], foo[1][0]) would fail
+}
+
 fn vec() {
     let mut foo = vec![1, 2];
     let temp = foo[0];
@@ -54,6 +71,7 @@ fn main() {
     field();
     array();
     slice();
+    unswappable_slice();
     vec();
 
     let mut a = 42;
