@@ -6,6 +6,8 @@ use std::path::{PathBuf, Path};
 use std::process::Command;
 use std::ops::Not;
 
+const XARGO_MIN_VERSION: (u32, u32, u32) = (0, 3, 17);
+
 const CARGO_MIRI_HELP: &str = r#"Interprets bin crates and tests in Miri
 
 Usage:
@@ -91,7 +93,7 @@ fn list_targets() -> impl Iterator<Item=cargo_metadata::Target> {
     let mut metadata = if let Ok(metadata) = cmd.exec() {
         metadata
     } else {
-        show_error(format!("Could not obtain Cargo metadata"));
+        show_error(format!("Could not obtain Cargo metadata; likely an ill-formed manifest"));
     };
 
     let current_dir = std::env::current_dir();
@@ -113,7 +115,7 @@ fn list_targets() -> impl Iterator<Item=cargo_metadata::Target> {
                 package_manifest_directory == current_dir
             }
         })
-        .expect("could not find matching package");
+        .unwrap_or_else(|| show_error(format!("This seems to be a workspace, which is not supported by cargo-miri")));
     let package = metadata.packages.remove(package_index);
 
     // Finally we got the list of targets to build
@@ -136,8 +138,12 @@ fn test_sysroot_consistency() {
             .output().expect("Failed to run rustc to get sysroot info");
         let stdout = String::from_utf8(out.stdout).expect("stdout is not valid UTF-8");
         let stderr = String::from_utf8(out.stderr).expect("stderr is not valid UTF-8");
+        assert!(
+            out.status.success(),
+            "Bad status code {} when getting sysroot info via {:?}.\nstdout:\n{}\nstderr:\n{}",
+            out.status, cmd, stdout, stderr,
+        );
         let stdout = stdout.trim();
-        assert!(out.status.success(), "Bad status code when getting sysroot info.\nstdout:\n{}\nstderr:\n{}", stdout, stderr);
         PathBuf::from(stdout).canonicalize()
             .unwrap_or_else(|_| panic!("Failed to canonicalize sysroot: {}", stdout))
     }
@@ -254,25 +260,54 @@ fn setup(ask_user: bool) {
     }
 
     // First, we need xargo.
-    if xargo_version().map_or(true, |v| v < (0, 3, 16)) {
+    if xargo_version().map_or(true, |v| v < XARGO_MIN_VERSION) {
+        if std::env::var("XARGO").is_ok() {
+            // The user manually gave us a xargo binary; don't do anything automatically.
+            show_error(format!("Your xargo is too old; please upgrade to the latest version"))
+        }
         let mut cmd = cargo();
         cmd.args(&["install", "xargo", "-f"]);
         ask_to_run(cmd, ask_user, "install a recent enough xargo");
     }
 
-    // Then, unless `XARGO_RUST_SRC` is set, we also need rust-src.
-    // Let's see if it is already installed.
-    if std::env::var("XARGO_RUST_SRC").is_err() {
-        let sysroot = Command::new("rustc").args(&["--print", "sysroot"]).output()
-            .expect("failed to get rustc sysroot")
-            .stdout;
-        let sysroot = std::str::from_utf8(&sysroot).unwrap();
-        let src = Path::new(sysroot.trim_end_matches('\n')).join("lib").join("rustlib").join("src");
-        if !src.exists() {
-            let mut cmd = Command::new("rustup");
-            cmd.args(&["component", "add", "rust-src"]);
-            ask_to_run(cmd, ask_user, "install the rustc-src component for the selected toolchain");
+    // Determine where the rust sources are located.  `XARGO_RUST_SRC` env var trumps everything.
+    let rust_src = match std::env::var("XARGO_RUST_SRC") {
+        Ok(val) => PathBuf::from(val),
+        Err(_) => {
+            // Check for `rust-src` rustup component.
+            let sysroot = Command::new("rustc").args(&["--print", "sysroot"]).output()
+                .expect("failed to get rustc sysroot")
+                .stdout;
+            let sysroot = std::str::from_utf8(&sysroot).unwrap();
+            let sysroot = Path::new(sysroot.trim_end_matches('\n'));
+            // First try: `$SYSROOT/lib/rustlib/src/rust`; test if that contains `Cargo.lock`.
+            let rustup_src = sysroot.join("lib").join("rustlib").join("src").join("rust");
+            let base_dir = if rustup_src.join("Cargo.lock").exists() {
+                // Just use this.
+                rustup_src
+            } else {
+                // Maybe this is a local toolchain built with `x.py` and linked into `rustup`?
+                // Second try: `$SYSROOT/../../..`; test if that contains `x.py`.
+                let local_src = sysroot.parent().and_then(Path::parent).and_then(Path::parent);
+                match local_src {
+                    Some(local_src) if local_src.join("x.py").exists() => {
+                        // Use this.
+                        PathBuf::from(local_src)
+                    }
+                    _ => {
+                        // Fallback: Ask the user to install the `rust-src` component, and use that.
+                        let mut cmd = Command::new("rustup");
+                        cmd.args(&["component", "add", "rust-src"]);
+                        ask_to_run(cmd, ask_user, "install the rustc-src component for the selected toolchain");
+                        rustup_src
+                    }
+                }
+            };
+            base_dir.join("src") // Xargo wants the src-subdir
         }
+    };
+    if !rust_src.exists() {
+        show_error(format!("Given Rust source directory `{}` does not exist.", rust_src.display()));
     }
 
     // Next, we need our own libstd. We will do this work in whatever is a good cache dir for this platform.
@@ -288,9 +323,7 @@ fn setup(ask_user: bool) {
 default_features = false
 # We need the `panic_unwind` feature because we use the `unwind` panic strategy.
 # Using `abort` works for libstd, but then libtest will not compile.
-# FIXME: Temporarily enabling backtrace feature to work around
-# <https://github.com/rust-lang/rust/issues/64410>.
-features = ["panic_unwind", "backtrace"]
+features = ["panic_unwind"]
 
 [dependencies.test]
         "#).unwrap();
@@ -308,12 +341,13 @@ path = "lib.rs"
     File::create(dir.join("lib.rs")).unwrap();
     // Prepare xargo invocation.
     let target = get_arg_flag_value("--target");
-    let print_env = !ask_user && has_arg_flag("--env"); // whether we just print the necessary environment variable
+    let print_sysroot = !ask_user && has_arg_flag("--print-sysroot"); // whether we just print the sysroot path
     let mut command = xargo();
     command.arg("build").arg("-q");
     command.current_dir(&dir);
     command.env("RUSTFLAGS", miri::miri_default_args().join(" "));
-    command.env("XARGO_HOME", dir.to_str().unwrap());
+    command.env("XARGO_HOME", &dir);
+    command.env("XARGO_RUST_SRC", &rust_src);
     // In bootstrap, make sure we don't get debug assertons into our libstd.
     command.env("RUSTC_DEBUG_ASSERTIONS", "false");
     // Handle target flag.
@@ -337,8 +371,9 @@ path = "lib.rs"
     };
     let sysroot = if is_host { dir.join("HOST") } else { PathBuf::from(dir) };
     std::env::set_var("MIRI_SYSROOT", &sysroot); // pass the env var to the processes we spawn, which will turn it into "--sysroot" flags
-    if print_env {
-        println!("MIRI_SYSROOT={}", sysroot.display());
+    if print_sysroot {
+        // Print just the sysroot and nothing else; this way we do not need any escaping.
+        println!("{}", sysroot.display());
     } else if !ask_user {
         println!("A libstd for Miri is now available in `{}`.", sysroot.display());
     }
