@@ -12,9 +12,9 @@ fn simple2() {
     assert_ne!(42, 24);
 }
 
-// A test that won't work on miri (tests disabling tests)
-#[cfg(not(miri))]
+// A test that won't work on miri (tests disabling tests).
 #[test]
+#[cfg_attr(miri, ignore)]
 fn does_not_work_on_miri() {
     let x = 0u8;
     assert!(&x as *const _ as usize % 4 < 4);
@@ -41,3 +41,23 @@ fn entropy_rng() {
 fn num_cpus() {
     assert_eq!(num_cpus::get(), 1);
 }
+
+
+// FIXME: Remove this `cfg` once we fix https://github.com/rust-lang/miri/issues/1059.
+// We cfg-gate the `should_panic` attribute and the `panic!` itself, so that the test
+// stdout does not depend on the platform.
+#[test]
+#[cfg_attr(not(windows), should_panic(expected="Explicit panic"))]
+fn do_panic() { // In large, friendly letters :)
+    #[cfg(not(windows))]
+    panic!("Explicit panic from test!");
+}
+
+// FIXME: see above
+#[test]
+#[allow(const_err)]
+#[cfg_attr(not(windows), should_panic(expected="the len is 0 but the index is 42"))]
+fn fail_index_check() {
+    #[cfg(not(windows))]
+    [][42]
+}
