@@ -3,5 +3,5 @@
 fn main() {
     // With the nested Vec, this is calling Offset(Unique::empty(), 0) on drop.
     let args : Vec<Vec<i32>> = Vec::new();
-    let _ = box args;
+    let _val = box args;
 }
