@@ -9,10 +9,12 @@ extern crate syntax;
 
 use rustc::session::config::ErrorOutputType;
 use rustc::session::early_error;
-#[cfg(any(feature = "clippy", feature = "ipc"))]
+#[cfg(feature = "ipc")]
 use rustc_driver::Compilation;
 use rustc_driver::{run_compiler, Callbacks};
 use rustc_interface::interface;
+#[cfg(feature = "ipc")]
+use rustc_interface::Queries;
 
 use std::env;
 #[allow(unused_imports)]
@@ -90,22 +92,22 @@ impl Callbacks for ShimCalls {
     fn config(&mut self, config: &mut interface::Config) {
         config.opts.debugging_opts.continue_parse_after_error = true;
         config.opts.debugging_opts.save_analysis = true;
-    }
 
-    #[cfg(feature = "clippy")]
-    fn after_parsing(&mut self, compiler: &interface::Compiler) -> Compilation {
+        #[cfg(feature = "clippy")]
         match self.clippy_preference {
             Some(preference) if preference != clippy::ClippyPreference::Off => {
-                clippy::after_parse_callback(compiler);
+                clippy::config(config);
             }
             _ => {}
         }
-
-        Compilation::Continue
     }
 
     #[cfg(feature = "ipc")]
-    fn after_expansion(&mut self, compiler: &interface::Compiler) -> Compilation {
+    fn after_expansion<'tcx>(
+        &mut self,
+        compiler: &interface::Compiler,
+        queries: &'tcx Queries<'tcx>,
+    ) -> Compilation {
         use rustc::session::config::Input;
 
         use futures::future::Future;
@@ -119,7 +121,7 @@ impl Callbacks for ShimCalls {
 
         let sess = compiler.session();
         let input = compiler.input();
-        let crate_name = compiler.crate_name().unwrap().peek().clone();
+        let crate_name = queries.crate_name().unwrap().peek().clone();
 
         let cwd = &sess.working_dir.0;
 
@@ -152,7 +154,11 @@ impl Callbacks for ShimCalls {
     }
 
     #[cfg(feature = "ipc")]
-    fn after_analysis(&mut self, compiler: &interface::Compiler) -> Compilation {
+    fn after_analysis<'tcx>(
+        &mut self,
+        compiler: &interface::Compiler,
+        queries: &'tcx Queries<'tcx>,
+    ) -> Compilation {
         use futures::future::Future;
 
         let callbacks = match self.callbacks.as_ref() {
@@ -163,12 +169,12 @@ impl Callbacks for ShimCalls {
         use rustc_save_analysis::CallbackHandler;
 
         let input = compiler.input();
-        let crate_name = compiler.crate_name().unwrap().peek().clone();
+        let crate_name = queries.crate_name().unwrap().peek().clone();
 
         // Guaranteed to not be dropped yet in the pipeline thanks to the
         // `config.opts.debugging_opts.save_analysis` value being set to `true`.
-        let expanded_crate = &compiler.expansion().unwrap().peek().0;
-        compiler.global_ctxt().unwrap().peek_mut().enter(|tcx| {
+        let expanded_crate = &queries.expansion().unwrap().peek().0;
+        queries.global_ctxt().unwrap().peek_mut().enter(|tcx| {
             // There are two ways to move the data from rustc to the RLS, either
             // directly or by serialising and deserialising. We only want to do
             // the latter when there are compatibility issues between crates.
