@@ -5,10 +5,13 @@
 //! functionality such as 'goto definition', symbol search, reformatting, and
 //! code completion, and enables renaming and refactorings.
 
-#![feature(rustc_private, integer_atomics, drain_filter, hash_raw_entry)]
-#![allow(unknown_lints)]
-#![warn(clippy::all, rust_2018_idioms)]
-#![allow(clippy::cyclomatic_complexity, clippy::too_many_arguments)]
+#![feature(rustc_private, drain_filter)]
+#![warn(rust_2018_idioms)]
+#![warn(clippy::all, clippy::clone_on_ref_ptr)]
+#![allow(clippy::cognitive_complexity, clippy::too_many_arguments, clippy::redundant_closure)]
+
+#[macro_use]
+extern crate failure;
 
 pub use rls_analysis::{AnalysisHost, Target};
 pub use rls_vfs::Vfs;
