@@ -1,6 +1,6 @@
 // A generic trait to abstract the rewriting of an element (of the AST).
 
-use std::cell::RefCell;
+use std::cell::{Cell, RefCell};
 use std::rc::Rc;
 
 use syntax::parse::ParseSess;
@@ -29,17 +29,17 @@ pub(crate) struct RewriteContext<'a> {
     pub(crate) parse_session: &'a ParseSess,
     pub(crate) source_map: &'a SourceMap,
     pub(crate) config: &'a Config,
-    pub(crate) inside_macro: Rc<RefCell<bool>>,
+    pub(crate) inside_macro: Rc<Cell<bool>>,
     // Force block indent style even if we are using visual indent style.
-    pub(crate) use_block: RefCell<bool>,
+    pub(crate) use_block: Cell<bool>,
     // When `is_if_else_block` is true, unindent the comment on top
     // of the `else` or `else if`.
-    pub(crate) is_if_else_block: RefCell<bool>,
+    pub(crate) is_if_else_block: Cell<bool>,
     // When rewriting chain, veto going multi line except the last element
-    pub(crate) force_one_line_chain: RefCell<bool>,
+    pub(crate) force_one_line_chain: Cell<bool>,
     pub(crate) snippet_provider: &'a SnippetProvider<'a>,
     // Used for `format_snippet`
-    pub(crate) macro_rewrite_failure: RefCell<bool>,
+    pub(crate) macro_rewrite_failure: Cell<bool>,
     pub(crate) report: FormatReport,
     pub(crate) skip_context: SkipContext,
     pub(crate) skipped_range: Rc<RefCell<Vec<(usize, usize)>>>,
@@ -47,7 +47,7 @@ pub(crate) struct RewriteContext<'a> {
 
 pub(crate) struct InsideMacroGuard {
     is_nested_macro_context: bool,
-    inside_macro_ref: Rc<RefCell<bool>>,
+    inside_macro_ref: Rc<Cell<bool>>,
 }
 
 impl InsideMacroGuard {
@@ -69,7 +69,7 @@ impl<'a> RewriteContext<'a> {
 
     /// Returns `true` if we should use block indent style for rewriting function call.
     pub(crate) fn use_block_indent(&self) -> bool {
-        self.config.indent_style() == IndentStyle::Block || *self.use_block.borrow()
+        self.config.indent_style() == IndentStyle::Block || self.use_block.get()
     }
 
     pub(crate) fn budget(&self, used_width: usize) -> usize {
@@ -77,7 +77,7 @@ impl<'a> RewriteContext<'a> {
     }
 
     pub(crate) fn inside_macro(&self) -> bool {
-        *self.inside_macro.borrow()
+        self.inside_macro.get()
     }
 
     pub(crate) fn enter_macro(&self) -> InsideMacroGuard {
@@ -93,6 +93,6 @@ impl<'a> RewriteContext<'a> {
     }
 
     pub(crate) fn is_if_else_block(&self) -> bool {
-        *self.is_if_else_block.borrow()
+        self.is_if_else_block.get()
     }
 }
