@@ -615,3 +615,43 @@ fn issue3457() {
         }
     }
 }
+
+// #3498
+static REPRO: &[usize] = &[
+    #[cfg(feature = "zero")]
+    0,
+];
+
+fn overflow_with_attr() {
+    foo(
+        #[cfg(feature = "zero")]
+        0,
+    );
+    foobar(
+        #[cfg(feature = "zero")]
+        0,
+    );
+    foobar(
+        x,
+        y,
+        #[cfg(feature = "zero")]
+        {},
+    );
+}
+
+// https://github.com/rust-lang/rustfmt/issues/3765
+fn foo() {
+    async {
+        // Do
+        // some
+        // work
+    }
+    .await;
+
+    async {
+        // Do
+        // some
+        // work
+    }
+    .await;
+}
